<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;
use \Setforms2\Shared\ORM\DateTime;
use Teknisa\OPL\Modules\Consolidacao\Services\Sup07200LancamentoDePedidosService;

class Sup07200LancamentoDePedidosController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $newCode;
    protected $crudService;
    
    public function __construct($crudService, $parameterBag, Sup07200LancamentoDePedidosService $service, $environment, $newCode) {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
		$this->service      = $service;
		$this->environment  = $environment;
		$this->newCode      = $newCode;
    }
    
    public function retornaFloat($valor) {
        if(gettype($valor) === 'string') {
	        $valor = str_replace('.', '', $valor);
            $valor = str_replace(',', '.', $valor);
	        $valor = (float) $valor;
	    }
	    return $valor;
	}
    
    public function checkIgualdadeItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $result = $this->service->checkIgualdadeItemPedidoOL($row);
            
            if(isset($result[0]["CONTADOR"])) {
                if($result[0]["CONTADOR"] > 0) {
                    throw new \Exception("ITEM_ALREADY_EXISTS_PEDIDO", 500);
                }
            } else {
                throw new \Exception("ERRO_QUERY", 500);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function insereItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
            
            if($row["QTPROGENPEOL"] == 0) {
                throw new \Exception("QTPROGENPEOL_EQUALS_ZERO", 500);
            } else {
                $this->service->insereItemPedidoOL($row);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function excluiItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->excluiItemPedidoOL($row);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function alteraDataEntregaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $aux = 0;
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $row["OLD_DTPROGENPEOL"] = $row["DTPROGENPEOL"];
                $row["DTPROGENPEOL"] = $row["NEW_DTPROGENPEOL"];
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("DTPROGENPEOL_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraDataEntregaItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function alteraMarcaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $aux = 0;
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $row["CDMARCA"] = $row["NEW_CDMARCA"];
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("CDMARCA_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraMarcaItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function alteraProdutoItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $aux = 0;
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $row["CDPRODUTO"] = $row["NEW_CDPRODUTO"];
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["AJUSTE_UNIDADE"] = self::retornaFloat($row["AJUSTE_UNIDADE"]);
                $row["AJUSTE_QUANTIDADE"] = self::retornaFloat($row["AJUSTE_QUANTIDADE"]);
                
                $QTDAJUSTADA = round(($row["QTPROGENPEOL"] + (($row["AJUSTE_UNIDADE"] / 100) * $row["QTPROGENPEOL"])) + 
                    (($row["AJUSTE_QUANTIDADE"] / 100) * ($row["QTPROGENPEOL"] + (($row["AJUSTE_UNIDADE"] / 100) * $row["QTPROGENPEOL"]))), 3);
                    
                $row["NEW_QTPROGENPEOL"] = $QTDAJUSTADA;
                    
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("CDPRODUTO_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraProdutoItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function alteraQuantidadeItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["NEW_QTPROGENPEOL"] = self::retornaFloat($row["NEW_QTPROGENPEOL"]);
                $row["OLD_QTPROGENPEOL"] = self::retornaFloat($row["OLD_QTPROGENPEOL"]);
                
                $this->service->alteraQuantidadeItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function incluiItemPedidoLote(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            $this->service->inclusaoDePedidoParaLote($rows[0]);
            
            foreach($rows as $row) {
                $this->service->incluiItemPedidoLote($row);
                $this->service->ajusteItemPedidoLote($row);
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function incluiItemPedidoLoteDiversos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $this->service->inclusaoDePedidoParaLote($row);
                $this->service->incluiItemPedidoLote($row);
                $this->service->ajusteItemPedidoLote($row);
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function inclusaoReservaItemPedido(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            $aux = 0;
            
            foreach($rows as $row) {
                $QUANTESTOQ = null;
                $QUANTRESRV = null;
                $QUANTITEMP = null;
                $CDOLFILIAL = $this->service->verificaOLQueAtendeFilial($row);
                $QUANTESTOQ = $this->service->posicaoEstoqueItemOL($CDOLFILIAL, $row);
                $QUANTRESRV = $this->service->verificaReservaDemaisPedidos($row);
                $QUANTITEMP = self::retornaFloat($row["QTPROGENPEOL"]);
                
                $QTRESERVESTOL = (($QUANTESTOQ - $QUANTRESRV) - $QUANTITEMP);
                
                if($QTRESERVESTOL >= 0) {
                    $this->service->incluiReservaItemPedidoOL($row, $QTRESERVESTOL);
                } else {
                    $aux++;
                }
            }
            
            if($aux == 0) {
                $this->service->realizaCommit();
            } else {
                $this->service->realizaRollback();
                throw new \Exception("QTD_INDISPONIVEL_RESERVA", 500);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function exclusaoReservaItemPedido(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function baixaManualItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["QTORIGPRODPED"] = self::retornaFloat($row["QTORIGPRODPED"]);
                $this->service->atualizaSituacaoItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function checkOperBlockUnblockPer(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $result = $this->service->checkOperBlockUnblockPer($row);
            
            if(isset($result[0]["CDOPERADOR"])) {
                $response->addDataSet(new DataSet("CDOPERADOR", $result[0]));
            } else {
                
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function desbloqueioBloqueioItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->beginTransaction();
            
            foreach($rows as $row) {
                $this->service->desbloqueioBloqueioItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
}