<?php
namespace Teknisa\OPL\Modules\Consolidacao\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Sup07200LancamentoDePedidosService {
    protected $connection;
    
    public function __construct($entityManager, $environment) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    public function realizaCommit() {
        $this->entityManager->commit();
    }
    
    public function realizaRollback() {
        $this->entityManager->rollBack();
    }
    
    public function beginTransaction() {
        $this->entityManager->beginTransaction();
    }
    
    public function checkIgualdadeItemPedidoOL($row) {
        $param = array();
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        
        $query = "SELECT
                    COUNT(*) CONTADOR
                  FROM
                    ITEMPEDEXOL
                  WHERE
                    NRPEDEXOL = :P_NRPEDEXOL
                    AND DTPROGENPEOL = TO_DATE(:P_DTPROGENPEOL, 'DD/MM/YYYY')
                    AND CDMARCA = :P_CDMARCA
                    AND CDPRODUTO = :P_CDPRODUTO";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function insereItemPedidoOL($row) {
        $param = array();
        $param["CDFILDEST"]         = $row["CDFILDEST"];
        $param["CDFILLANC"]         = $row["CDFILLANC"];
        $param["NRAUTOFORN"]        = $row["NRPEDEXOL"];
        $param["CDPRODUTO"]         = $row["CDPRODUTO"];
        $param["QTPROGENTAF"]       = $row["QTPROGENPEOL"];
        $param["QTEFETENTRAF"]      = null;
        $param["DTPROGENTAF"]       = $row["DTPROGENPEOL"];
        $param["DTEFETENTAF"]       = null;
        $param["CDMARCA"]           = $row["CDMARCA"];
        $param["CDLOTENVITOL"]      = $row["CDLOTENVITOL"];
        $param["IDBAIXITMAF"]       = null;
        $param["IDBAIXITAFS"]       = null;
        $param["TXOBSITMAF"]        = isset($row["TXOBSITMPEOL"]) ? $row["TXOBSITMPEOL"] : null;
        $param["CDOPERBAIXA"]       = null;
        $param["DTBAIXITMAF"]       = null;
        $param["QTDORIGINAL"]       = $row["QTPROGENPEOL"];
        $param["CDPRODESTO"]        = $row["CDPRODESTO"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDARVPROD"]       = $row["CDARVPROD"];
        $param["CDRASTRITM"]        = null;
        $param["VRPREUNITPED"]      = null;
        
        $query = "INSERT INTO ITEMPEDEXOL
                      (CDFILDEST,    CDFILLANC,    NRPEDEXOL,    CDPRODUTO,     QTPROGENPEOL,   QTEFETENPEOL,
                       DTPROGENPEOL, DTEFETENPEOL, CDMARCA,      CDLOTENVITOL,  IDBAIXENPEOL,   IDBAIXAOLSAI,
                       TXOBSITMPEOL, CDOPERBAIXA,  DTBAIXTMPEOL, QTORIGPRODPED, CDPRODORIGPEDOL,
                       DTPROGEMBARQ, CDRASTRITM, VRPREUNITPED)
                    SELECT :CDFILDEST,   :CDFILLANC,   :NRAUTOFORN,  :CDPRODUTO,    :QTPROGENTAF, :QTEFETENTRAF,
                           :DTPROGENTAF, :DTEFETENTAF, :CDMARCA,     :CDLOTENVITOL, :IDBAIXITMAF, :IDBAIXITAFS,
                           :TXOBSITMAF,  :CDOPERBAIXA, :DTBAIXITMAF, :QTDORIGINAL,  :CDPRODESTO,
                           CALCULA_DATAEMBARQUE_OL(:P_CDFILIOPELOG, :CDFILDEST, :P_CDARVPROD, :DTPROGENTAF), :CDRASTRITM,
                           :VRPREUNITPED
                      FROM PEDIDOEXOL P
                     WHERE P.CDFILDEST = :CDFILDEST
                       AND P.CDFILLANC = :CDFILLANC
                       AND P.NRPEDEXOL = :NRAUTOFORN
                       AND NVL(P.IDBAIXEXSAI, '1') <> '2'";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function excluiItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "DELETE FROM ITEMPEDEXOL
                    WHERE
                      CDFILDEST = :OLD_CDFILDEST AND
                      CDFILLANC = :OLD_CDFILLANC AND
                      NRPEDEXOL = :OLD_NRAUTOFORN AND
                      CDPRODUTO = :OLD_CDPRODUTO AND
                      DTPROGENPEOL = :OLD_DTPROGENTAF AND
                      CDMARCA = :OLD_CDMARCA AND
                      NVL(IDBAIXAOLSAI, '1') <> '2'";
                      
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraDataEntregaItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["DTPROGENTAF"]       = $row["NEW_DTPROGENPEOL"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["OLD_DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        DTPROGENPEOL = :DTPROGENTAF
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraMarcaItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["CDMARCA"]           = $row["NEW_CDMARCA"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["OLD_CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        CDMARCA = :CDMARCA
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraProdutoItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["CDPRODUTO"]         = $row["NEW_CDPRODUTO"];
        $param["CDPRODESTO"]        = $row["NEW_CDPRODESTO"];
        $param["QTPROGENPEOL"]      = $row["NEW_QTPROGENPEOL"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["OLD_CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        CDPRODUTO = :CDPRODUTO,
                        CDPRODORIGPEDOL = :CDPRODESTO,
                        QTPROGENPEOL = :QTPROGENPEOL
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraQuantidadeItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["QTPROGENPEOL"]      = $row["NEW_QTPROGENPEOL"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        QTPROGENPEOL = :QTPROGENPEOL
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function pegaUltimoNumeroPedido($CDFILLANC, $CDFILDEST) {
        $param = array();
        $param["P_CDFILLANC"] = $CDFILLANC;
        $param["P_CDFILDEST"] = $CDFILDEST;
        
        $query = "SELECT NRPEDEXOL FROM PEDIDOEXOL
                    WHERE ROWNUM = 1 
                    AND CDFILDEST = :P_CDFILDEST 
                    AND CDFILLANC = :P_CDFILLANC 
                    ORDER BY NRPEDEXOL DESC";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            $return  = $return[0]["NRPEDEXOL"];
            return $return;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function inclusaoDePedidoParaLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $row["NEW_NRPEDEXOL"] = self::pegaUltimoNumeroPedido($row["CDFILLANC"], $CDFILDEST[0]);
        $row["NEW_NRPEDEXOL"] = intval($row["NEW_NRPEDEXOL"]);
        $row["NEW_NRPEDEXOL"]++;
        $row["NEW_NRPEDEXOL"] = str_pad($row["NEW_NRPEDEXOL"], 10, 0, STR_PAD_LEFT);
        
        var_dump($row["NEW_NRPEDEXOL"]);
        
        $param = array();
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_NRPEDEXOL_NEW"]   = $row["NEW_NRPEDEXOL"];
        
        $query = "INSERT INTO PEDIDOEXOL
                  (CDFILDEST, CDFILLANC, NRPEDEXOL, DTPEDEXOL,
                   IDEMITEXOL, IDTPFRETEXOL, CDEMPRESA, CDTIPCOB,
                   IDTPJTREXOL,  NRINSJTREXOL, CDTPCTPAGAR, CDFILIOPELOG, IDTIPOEXOL, IDPEDEXOLEXTR)
                  (SELECT P.CDFILDEST, P.CDFILLANC, :P_NRPEDEXOL_NEW, TRUNC(SYSDATE),
                          P.IDEMITEXOL, P.IDTPFRETEXOL, P.CDEMPRESA, P.CDTIPCOB,
                          P.IDTPJTREXOL,  P.NRINSJTREXOL, P.CDTPCTPAGAR, P.CDFILIOPELOG, P.IDTIPOEXOL, P.IDPEDEXOLEXTR
                    FROM PEDIDOEXOL P
                   WHERE P.CDFILDEST    = :P_CDFILDEST
                     AND P.CDFILLANC    = :P_CDFILLANC
                     AND P.NRPEDEXOL    = :P_NRPEDEXOL
                     AND NVL(P.IDBAIXEXSAI, '1') <> '2')";
                     
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function incluiItemPedidoLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $row["NEW_NRPEDEXOL"] = self::pegaUltimoNumeroPedido($row["CDFILLANC"], $CDFILDEST[0]);
        
        var_dump($row["NEW_NRPEDEXOL"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDOPERADOR"]      = $this->environment->getCdOperador();
        $param["P_CDLOTENVITOL"]    = $row["NEW_CDLOTENVITOL"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_NRPEDEXOL_NEW"]   = $row["NEW_NRPEDEXOL"];
        
        $query = "INSERT INTO ITEMPEDEXOL
                      (CDFILDEST, CDFILLANC, NRPEDEXOL, CDPRODUTO, QTPROGENPEOL, QTEFETENPEOL,
                       DTPROGENPEOL, DTEFETENPEOL,  CDMARCA, CDLOTENVITOL,
                       IDBAIXENPEOL, IDBAIXAOLSAI, TXOBSITMPEOL, CDOPERBAIXA, DTBAIXTMPEOL, DTPROGEMBARQ, CDOPERBLOQITM, CDRASTRITM)
                      (SELECT I.CDFILDEST, I.CDFILLANC, :P_NRPEDEXOL_NEW, I.CDPRODUTO, I.QTPROGENPEOL, I.QTEFETENPEOL,
                              I.DTPROGENPEOL, I.DTEFETENPEOL, I.CDMARCA, :P_CDLOTENVITOL,
                              I.IDBAIXENPEOL, I.IDBAIXAOLSAI, I.TXOBSITMPEOL, I.CDOPERBAIXA,
                              I.DTBAIXTMPEOL, I.DTPROGEMBARQ, :P_CDOPERADOR, I.CDRASTRITM
                        FROM ITEMPEDEXOL I
                       WHERE I.CDFILDEST    = :P_CDFILDEST
                         AND I.CDFILLANC    = :P_CDFILLANC
                         AND I.NRPEDEXOL    = :P_NRPEDEXOL
                         AND I.CDPRODUTO    = :P_CDPRODUTO
                         AND DTPROGENPEOL   = :P_DTPROGENPEOL
                         AND I.CDMARCA      = :P_CDMARCA)";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function ajusteItemPedidoLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                   SET IDITEMAJUSTE = 'S'
                 WHERE CDFILDEST    = :P_CDFILDEST AND
                  CDFILLANC    = :P_CDFILLANC AND
                  NRPEDEXOL    = :P_NRPEDEXOL AND
                  CDPRODUTO    = :P_CDPRODUTO AND
                  DTPROGENPEOL = :P_DTPROGENPEOL AND
                  CDMARCA      = :P_CDMARCA";
                  
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function excluiReservaItemPedidoLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        
        $query = "DELETE FROM RESERVAPEDEXOL
                     WHERE CDFILDEST     = :P_CDFILDEST
                       AND CDFILLANC     = :P_CDFILLANC
                       AND NRPEDEXOL     = :P_NRPEDEXOL
                       AND CDPRODUTO     = :P_CDPRODUTO
                       AND CDMARCA       = :P_CDMARCA
                       AND DTPROGENPEOL  = :P_DTPROGENPEOL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function verificaOLQueAtendeFilial($row) {
        $param = array();
        $param["P_CDFILIOPELOG"] = $row["CDFILIOPELOG"];
        
        $query = "SELECT NVL(MAX(CDFILIAL), :P_CDFILIOPELOG) CDFILIAL
                     FROM FILIAL
                    WHERE CDFILIOLAGETERC = :P_CDFILIOPELOG";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            $return = $return[0]["CDFILIAL"];
            return $return;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function posicaoEstoqueItemOL($CDOLFILIAL, $row) {
        $param = array();
        $param["P_CDFILIOPELOG"]    = $CDOLFILIAL;
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        
        $query = "SELECT CALC_ESTOQ_CONS(:P_CDFILIOPELOG, Q.CDPRODUTO, Q.DTPOSESTCON) QUANT_ESTOQUE
                  FROM (SELECT P.CDPRODUTO, NVL(MAX(DTPOSESTCON), SYSDATE) AS DTPOSESTCON
                          FROM (SELECT DECODE (R.CDPRODARVOL, P.CDPRODESTO, R.CDPRODARVOL, PR.CDPRODUTO) AS CDPRODUTO
                                  FROM RELACPRODOL R, PRODUTO P, PRODUTO PR
                                 WHERE P.CDPRODUTO   = R.CDPRODARVOL
                                   AND P.CDPRODESTO  = PR.CDPRODUTO
                                   AND R.CDPRARVORIG = :P_CDPRODUTO
                                   AND R.CDMARCA     = :P_CDMARCA) P, POSESTCONS E
                         WHERE E.CDFILIAL  = :P_CDFILIOPELOG
                           AND E.CDPRODUTO = P.CDPRODUTO
                         GROUP BY P.CDPRODUTO) Q";
                         
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0]["QUANT_ESTOQUE"];
            else
                return null;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function verificaReservaDemaisPedidos($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_NRAUTOFORN"]      = $row["NRPEDEXOL"];
        $param["P_DTPROGENTAF"]     = $row["DTPROGENPEOL"];
        
        $query = "SELECT SUM(QTRESERVESTOL) ESTOQ_RESERVADO
                      FROM RESERVAPEDEXOL
                     WHERE CDPRODUTO    =  :P_CDPRODUTO
                       AND CDMARCA      =  :P_CDMARCA
                       AND CDFILRESOL   =  :P_CDFILIOPELOG
                       AND (CDFILDEST   <> :P_CDFILDEST
                        OR CDFILLANC    <> :P_CDFILLANC
                        OR NRPEDEXOL    <> :P_NRAUTOFORN
                        OR DTPROGENPEOL <> :P_DTPROGENTAF)";
                         
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0]["ESTOQ_RESERVADO"];
            else
                return null;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function incluiReservaItemPedidoOL($row, $QTRESERVESTOL) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_QTRESERVESTOL"]   = $QTRESERVESTOL;
        
        $query = "INSERT INTO RESERVAPEDEXOL (CDFILDEST, CDFILLANC, NRPEDEXOL, CDPRODUTO, CDMARCA,
                                                DTPROGENPEOL, QTRESERVESTOL, CDFILRESOL, DTRESEMBARQ)
                    SELECT :P_CDFILDEST, :P_CDFILLANC, :P_NRPEDEXOL, :P_CDPRODUTO, :P_CDMARCA,
                           :P_DTPROGENPEOL, :P_QTRESERVESTOL, :P_CDFILIOPELOG,
                           CALCULA_DATAEMBARQUE_OL(:P_CDFILIOPELOG, :P_CDFILDEST,
                           :P_CDPRODUTO,  :P_DTPROGENPEOL)
                      FROM DUAL
                     WHERE NOT EXISTS (SELECT 1
                                         FROM RESERVAPEDEXOL R
                                        WHERE R.CDFILDEST    = :P_CDFILDEST
                                          AND R.CDFILLANC    = :P_CDFILLANC
                                          AND R.NRPEDEXOL    = :P_NRPEDEXOL
                                          AND R.CDPRODUTO    = :P_CDPRODUTO
                                          AND R.CDMARCA      = :P_CDMARCA
                                          AND R.DTPROGENPEOL = :P_DTPROGENPEOL)";
                                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaSituacaoItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["QTPROGENPEOL"]      = $row["QTPROGENPEOL"];
        $param["IDBAIXAOLSAI"]      = $row["NEW_IDBAIXAOLSAI"];
        $param["CDOPERBAIXA"]       = $row["NEW_CDOPERBAIXA"];
        $param["DTBAIXTMPEOL"]      = $row["NEW_DTBAIXTMPEOL"];
        $param["QTORIGPRODPED"]     = $row["QTORIGPRODPED"];
        $param["CDFILDEST"]         = $CDFILDEST[0];
        $param["CDFILLANC"]         = $row["CDFILLANC"];
        $param["NRPEDEXOL"]         = $row["NRPEDEXOL"];
        $param["CDPRODUTO"]         = $row["CDPRODUTO"];
        $param["CDMARCA"]           = $row["CDMARCA"];
        $param["DTPROGENPEOL"]      = $row["DTPROGENPEOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                       SET QTPROGENPEOL  = NVL(:QTPROGENPEOL, QTPROGENPEOL),
                           IDBAIXAOLSAI  = NVL(:IDBAIXAOLSAI, IDBAIXAOLSAI),
                           CDOPERBAIXA   = NVL(:CDOPERBAIXA ,  CDOPERBAIXA),
                           DTBAIXTMPEOL  = NVL(:DTBAIXTMPEOL, DTBAIXTMPEOL),
                           QTORIGPRODPED = :QTORIGPRODPED
                     WHERE CDFILDEST     = :CDFILDEST
                       AND CDFILLANC     = :CDFILLANC
                       AND NRPEDEXOL     = :NRPEDEXOL
                       AND CDPRODUTO     = :CDPRODUTO
                       AND CDMARCA       = :CDMARCA
                       AND DTPROGENPEOL  = :DTPROGENPEOL
                       AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                  
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function checkOperBlockUnblockPer($row) {
        $param = array();
        $param["P_CDOPERADOR"] = $row["CDOPERADOR"];
        
        $query = "SELECT O.CDOPERADOR 
                      FROM OPERGRUPOP O, AUTGRUPOP A
                     WHERE O.CDOPERADOR  = :P_CDOPERADOR
                       AND O.CDGRUPOPER  = A.CDGRUPOPER
                       AND A.IDAUTGRUPOP = 'BLOL'";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function desbloqueioBloqueioItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_IDITMPEDOLBLOQ"]  = $row["NEW_IDITMPEDOLBLOQ"];
        $param["P_DTBLOQITMPEDOL"]  = $row["NEW_DTBLOQITMPEDOL"];
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDOPERADOR"]      = $row["NEW_CDOPERADOR"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                       SET IDITMPEDOLBLOQ = :P_IDITMPEDOLBLOQ,
                           DTBLOQITMPEDOL = :P_DTBLOQITMPEDOL,
                           CDOPERBLOQITM  = :P_CDOPERADOR
                     WHERE CDFILDEST     = :P_CDFILDEST
                       AND CDFILLANC     = :P_CDFILLANC
                       AND NRPEDEXOL     = :P_NRPEDEXOL
                       AND CDMARCA       = :P_CDMARCA
                       AND CDPRODUTO     = :P_CDPRODUTO
                       AND DTPROGENPEOL  = :P_DTPROGENPEOL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
}